Global sMouseSensitivity:Float = 0.30
Global sMusicVolume:Double = 0.70
Global sSoundVolume:Double = 1.0
Global sFov:Int = 90
Global sWindowRes:Int = 3
Global sGameRes:Int = 1
Global sResNames:String[] = ["320X240", "640X480", "800X600", "960X720", "1440X1080"]
Global sResX:Int[] = [320, 640, 800, 960, 1440]
Global sResY:Int[] = [240, 480, 600, 720, 1080]

Type TInput Abstract
	Method IsDown() Abstract
	Method IsPressed() Abstract
	Method Name:String() Abstract
EndType

Type TKeyInput Extends TInput
	Field keyCode:Int
	Method IsDown()
		Return KeyDown(keyCode)
	End Method
	Method IsPressed()
		Return KeyHit(keyCode)
	End Method
	Method Name:String()
		Return NameForKeyCode(keyCode)
	End Method
	Function Make:TInput(keyCode:Int)
		Local k:TKeyInput = New TKeyInput
		k.keyCode = keyCode
		Return k
	End Function
EndType

Type TMouseInput Extends TInput
	Field mouseButton:Int
	Method IsDown()
		Return MouseDown(mouseButton)
	End Method
	Method IsPressed()
		Return MouseHit(mouseButton)
	End Method
	Method Name:String()
		Select mouseButton
			Case 1
				Return "LEFT CLICK"
			Case 2
				Return "RIGHT CLICK"
			Case 3
				Return "MIDDLE CLICK"
			Default
				Return "???"
		End Select
	End Method
	Function Make:TInput(mouseButton:Int)
		Local m:TMouseInput = New TMouseInput
		m.mouseButton = mouseButton
		Return m
	End Function
EndType

Const KEY_FORWARD:Int = 0
Const KEY_BACKWARD:Int = 1 
Const KEY_STRAFERIGHT:Int = 2
Const KEY_STRAFELEFT:Int = 3
Const KEY_USE:Int = 4
Const KEY_RUN:Int = 5
Const KEY_SELECT_SICKLE:Int = 6
Const KEY_SELECT_CHICKENCANNON:Int = 7
Const KEY_SELECT_GRENADELAUNCHER:Int =8
Const KEY_SELECT_PARUSU:Int = 9
Const KEY_SELECT_DOUBLEGRENADELAUNCHER:Int = 10
Const KEY_SELECT_SIGNOFMADNESS:Int = 11
Const KEY_SELECT_AIRHORN:Int = 12
Const KEY_TURN_LEFT:Int = 13
Const KEY_TURN_RIGHT:Int = 14
Const KEY_NAV_UP:Int = 15
Const KEY_NAV_DOWN:Int = 16
Const KEY_NAV_RIGHT:Int = 17
Const KEY_NAV_LEFT:Int = 18
Const KEY_CONFIRM:Int = 19
Const KEY_EXIT:Int = 20
Const KEY_MOUSELOCKTOGGLE:Int = 21
Const KEY_SHOOT:Int = 22
Const KEY_ALTFIRE:Int = 23
Global gInputs:TInput[] = [TKeyInput.Make(KEY_W), 
	TKeyInput.Make(KEY_S), 
	TKeyInput.Make(KEY_D), 
	TKeyInput.Make(KEY_A), 
	TKeyInput.Make(KEY_E), 
	TKeyInput.Make(KEY_LSHIFT), 
	TKeyInput.Make(KEY_1), 
	TKeyInput.Make(KEY_2), 
	TKeyInput.Make(KEY_3), 
	TKeyInput.Make(KEY_4), 
	TKeyInput.Make(KEY_5), 
	TKeyInput.Make(KEY_6), 
	TKeyInput.Make(KEY_7), 
	TKeyInput.Make(KEY_LEFT), 
	TKeyInput.Make(KEY_RIGHT), 
	TKeyInput.Make(KEY_UP), 
	TKeyInput.Make(KEY_DOWN), 
	TKeyInput.Make(KEY_RIGHT), 
	TKeyInput.Make(KEY_LEFT), 
	TKeyInput.Make(KEY_ENTER), 
	TKeyInput.Make(KEY_ESCAPE), 
	TKeyInput.Make(KEY_SEMICOLON), 
	TMouseInput.Make(MOUSE_LEFT), 
	TMouseInput.Make(MOUSE_RIGHT)]

Type TMenu Abstract
	Field options:String[]
	Field optionIndex:Int = 0
	Field optionsWindow:Int = -1
	Field fontWidth:Int = 16
	Field optionColors:Float[,]
	Field showCursor:Int = True
	Field cursorY:Float = 0.0
	Field y:Float = 104.0
	Field x:Float = 0.0
	Field showTitle:Int = True
	Method Init()
		optionColors = New Float[Len(options), 3]
		For Local i:Int = 0 To Len(options) - 1
			optionColors[i,0] = 0.0
			optionColors[i,1] = 0.0
			optionColors[i,2] = 1.0
		Next
	End Method
	Method Update()
		If InputPressed(KEY_NAV_DOWN) Then
			optionIndex :+ 1
			If optionIndex > Len(options) - 1 Then optionIndex = Len(options) - 1
			Play2DSound("menu0")
		ElseIf InputPressed(KEY_NAV_UP) Then
			optionIndex :- 1
			If optionIndex< 0 Then optionIndex = 0
			Play2DSound("menu0")
		EndIf
		Local cursorYTarget:Float = (optionIndex * fontWidth)
		If optionsWindow > -1 Then
			cursorYTarget = Min(cursorYTarget, Floor(optionsWindow/2.0) * fontWidth)
		EndIf
		cursorY:+ (cursorYTarget - cursorY) * 0.2
	End Method
	Method Draw()
		Local r:Float = 0.0
		Local g:Float = 0.0
		Local b:Float = 0.0
		Local oBegin:Int = 0
		Local oEnd:Int = Len(options)-1
		If optionsWindow > -1 Then
			oBegin = Max(0, optionIndex-Floor(optionsWindow/2.0))
			oEnd = Min(Len(options)-1, oBegin+optionsWindow-1)
		EndIf
		For Local i:Int = oBegin To oEnd
			If r <> optionColors[i,0] Or g <> optionColors[i,1] Or b <> optionColors[i,2] Then
				r = optionColors[i,0]
				g = optionColors[i,1]
				b = optionColors[i,2]
				glColor3f r,g,b
			EndIf
			DrawString(options[i], gFont, x + fontWidth, y + (i - oBegin) * fontWidth, fontWidth)
		Next
		If showCursor = True Then
			glColor3f 1.0, 1.0, 1.0
			glBindTexture GL_TEXTURE_2D, TTexture(MapValueForKey(gTextures, "titlecursor")).id
			DrawQuad(x, y + cursorY, fontWidth, y + fontWidth + cursorY)
		EndIf
		If InputPressed(KEY_EXIT) Then
			Play2DSound("menu1")
			If gMenu = gTitleMenu Or gMenu = gTitleMenu2 Then
				gGameState = STATE_QUITTING
			Else
				SetMenu(gTitleMenu)
			EndIf
			FlushKeys()
			FlushMouse()
		EndIf
	End Method
	Method Enter() Abstract
	Method Leave() Abstract
EndType

Type TTitleMenu Extends TMenu
	Field inGame:Int = False
	Method Update()
		Super.Update()
		If InputPressed(KEY_CONFIRM) Then
			Local idx = optionIndex
			If inGame = False Then idx :+ 1
			Select idx
				Case 0 'Resume
					gGameState = STATE_GAME
					Make3D()
				Case 1 'New Game
					SetMenu(gDifficultyMenu)
				Case 2 'Load Game
					SetMenu(gLoadMenu)
				Case 3 'Save Game
					SetMenu(gSaveMenu)
				Case 4 'Options
					SetMenu(gOptionsMenu)
				Case 5 'Controls
					SetMenu(gControlsMenu)
				Case 6 'Exit
					gGameState = STATE_QUITTING
			EndSelect
			Play2DSound("menu1")
		EndIf
	End Method
	Method Enter()
		
	End Method
	Method Leave()
		
	End Method
EndType

Type TOptionsMenu Extends TMenu
	Field newWinRes:Int = 0
	Field newGameRes:Int = 0
	Method Update()
		Super.Update()
		If InputPressed(KEY_NAV_LEFT) Then
			Select optionIndex
				Case 0
					sSoundVolume :- 0.10
				Case 1
					sMusicVolume :- 0.10
				Case 2
					sMouseSensitivity :- 0.10
				Case 3
					sFov :- 10
				Case 4
					newWinRes :- 1
					If newWinRes < newGameRes Then newWinRes = newGameRes
				Case 5
					newGameRes  :- 1
			EndSelect
		ElseIf InputPressed(KEY_NAV_RIGHT) Then
			Select optionIndex
				Case 0
					sSoundVolume :+ 0.1
				Case 1
					sMusicVolume :+ 0.1
				Case 2
					sMouseSensitivity :+ 0.1
				Case 3
					sFov :+ 10
				Case 4
					newWinRes :+ 1
				Case 5
					newGameRes  :+ 1
					If newGameRes > newWinRes Then newGameRes = newWinRes
			EndSelect
		EndIf
		
		sSoundVolume = Max(0.0, Min(1.0, sSoundVolume))
		sMusicVolume = Max(0.0, Min(1.0, sMusicVolume))
		sMouseSensitivity = Max(0.1, Min(9.0, sMouseSensitivity))
		sFov = Max(10.0, Min(120.0, sFov))		
		newWinRes = Max(0, Min(Len(sResNames) - 1, newWinRes))
		newGameRes = Max(0, Min(newWinRes, newGameRes))
		
		options[0] = "SOUND VOLUME: " + String.FromInt(Floor(sSoundVolume * 10.0) * 10) + "%"
		options[1] = "MUSIC VOLUME: " + String.FromInt(Floor(sMusicVolume * 10.0) * 10) + "%"
		options[2] = "SENSITIVITY: "  + Left(String.FromDouble(sMouseSensitivity), 3)
		options[3] = "FOV: " + String.FromInt(Floor(sFov))
		options[4] = "WINDOW:" + sResNames[newWinRes]
		options[5] = "RESOLUTION:" + sResNames[newGameRes]
		
		If InputPressed(KEY_CONFIRM) And optionIndex = Len(options) - 1 Then
			SetMenu(gTitleMenu)
			Play2DSound("menu1")
		EndIf
	End Method
	Method Enter()
		newWinRes = sWindowRes
		newGameRes = sGameRes
	End Method
	Method Leave()
		If newWinRes <> sWindowRes Then
			sWindowRes = newWinRes
			MakeWindow()
		EndIf
		sGameRes = newGameRes
		SaveSettings()
	End Method
EndType

Type TControlsMenu Extends TMenu
	Field anim:Int = 0
	Field frame:Int = 0
	Field rebinding:Int = 0 '1 for key, 2 for mouse
	Field toRebind:Int = -1
	Field nameBuffer:String = ""
	Field inputs: TInput[] = gInputs[..]
	Method New()
		options = ["FORWARD:W", 
			"BACKWARD:S", 
			"STRAFE RIGHT:D", 
			"STRAFE LEFT:A", 
			"USE:E", 
			"RUN:LEFT SHIFT", 
			"SICKLE:1", 
			"CHICKEN CANNON:2", 
			"GRENADE:3", 
			"PARUSU:4", 
			"DBL. GRENADE:5", 
			"SIGN OF MADNESS:6", 
			"AIRHORN:7", 
			"TURN LEFT:LEFT", 
			"TURN RIGHT:RIGHT", 
			"MENU UP:UP", 
			"MENU DOWN:DOWN", 
			"MENU LEFT:LEFT", 
			"MENU RIGHT:RIGHT", 
			"CONFIRM:ENTER", 
			"EXIT/CANCEL:ESCAPE", 
			"UNLOCK MOUSE:SEMICOLON", 
			"SHOOT:LEFT CLICK", 
			"ALT FIRE:RIGHT CLICK", 
			"ACCEPT CHANGES", 
			"CANCEL"]
		showTitle = False
		fontWidth = 12
		y = 80.0
		optionsWindow = 5
	End Method
	Method Update()
		If rebinding > 0 Then
			Repeat
				WaitEvent()
				Select EventID()
					Case EVENT_KEYDOWN
						If KeyHit(EventData()) Then
							inputs[optionIndex] = TKeyInput.Make(EventData())
							Exit
						EndIf
					Case EVENT_MOUSEDOWN
						If MouseHit(EventData()) Then
							inputs[optionIndex] = TMouseInput.Make(EventData())
							Exit
						EndIf
				End Select
			Forever
			options[optionIndex] = Left(nameBuffer, Instr(nameBuffer,":",1)) + inputs[optionIndex].Name()
			optionColors[optionIndex, 1] = 0.0
			rebinding = 0
			FlushKeys()
			FlushMouse()
		Else
			Super.Update()
			If InputPressed(KEY_CONFIRM) Then
				Play2DSound("menu1")
				If options[optionIndex] = "CANCEL" Then
					SetMenu(gTitleMenu)
				ElseIf options[optionIndex] = "ACCEPT CHANGES" Then
					gInputs = inputs[..]
					SetMenu(gTitleMenu)
				Else
					rebinding = 1
					nameBuffer = options[optionIndex]
					options[optionIndex] = "PRESS A KEY"
					optionColors[optionIndex, 1] = 1.0
					FlushKeys()
					FlushMouse()
				EndIf
			EndIf
		EndIf
	End Method
	Method Draw()
		Super.Draw()
		anim:+ 1
		If anim> 4 Then
			anim= 0
			frame:+ 1
			If frame> 3 Then frame= 0
		EndIf
		glColor4f 1.0, 1.0, 1.0, 1.0
		glBindTexture GL_TEXTURE_2D, TTexture(MapValueForKey(gTextures, "CTRLWRAITH_" + frame)).id
		DrawQuad(240.0, 152.0, 304.0, 216.0)
		DrawString("PRESS 'CONFIRM' ON A CONTROL, THEN", gFont, 8.0, 4.0, 8)
		DrawString("PRESS THE KEY OR MOUSE BUTTON TO BIND", gFont, 8.0, 12.0, 8)
		DrawString("SELECT 'ACCEPT CHANGES' AT THE", gFont, 8.0, 24.0, 8)
		DrawString("BOTTOM TO SAVE YOUR CHANGES", gFont, 8.0, 32.0, 8)
		If frame>1 Then
			glColor4f 1.0, 1.0, 0.0, 1.0
			DrawString("^", gFont, 160.0-8.0, 64.0, 8)
			DrawString("V", gFont, 160.0-8.0, 160.0, 8) 
		EndIf
	End Method
	Method Enter()
		For Local i:Int = 0 To Len(inputs)-1
			options[i] = Left(options[i], Instr(options[i],":",1)) + inputs[i].Name()
		Next
	End Method
	Method Leave()
		inputs = gInputs[..]
		SaveSettings()
	End Method
EndType

Type TEpisodeMenu Extends TMenu
	Method New()
		showTitle = False
	End Method
	Method Update()
		Super.Update()
		If InputPressed(KEY_CONFIRM) Then
			Play2DSound("menu1")
			gEpisodeNumber = optionIndex + 1
			gMapNumber = 1
			gGameState = STATE_GAME
			gGameInitialized = False
			ResetGame()
			InitGame()
			SetMenu(gTitleMenu)
		EndIf
	End Method
	Method Draw()
		Super.Draw()
		DrawString("SELECT EPISODE", gFont, 32.0, 32.0, 16)
	End Method
	Method Enter()
		
	End Method
	Method Leave()
		
	End Method
EndType

Type TDifficultyMenu Extends TMenu
	Method New()
		showTitle = False
	End Method
	Method Update()
		Super.Update()
		If InputPressed(KEY_CONFIRM) Then
			Play2DSound("menu1")
			gGameDifficulty = optionIndex
			SetMenu(gEpisodeMenu)
		EndIf
	End Method
	Method Draw()
		Super.Draw()
		DrawString("SELECT DIFFICULTY", gFont, 32.0, 32.0, 16)
	End Method
	Method Enter()
		
	End Method
	Method Leave()
		
	End Method
EndType

Const SAVE_FILES:Int = 0
Const LOAD_FILES:Int = 1
Type TFileMenu Extends TMenu
	Field kind:Int = SAVE_FILES
	Method New()
		y = 16.0
		showTitle = False
	End Method
	Method Update()
		Super.Update()
		If InputPressed(KEY_CONFIRM) Then
			If optionIndex = Len(options) -1 Then
				SetMenu(gTitleMenu)
				Play2DSound("menu1")
			ElseIf kind = SAVE_FILES And gGameInitialized = True Then
					Local file:TStream = WriteFile("assets/save" + optionIndex + ".sav")
					WriteLine file, gMapNumber
					WriteLine file, gEpisodeNumber
					WriteLine file, gGameDifficulty
					
					WriteLine file, gWeaponChicken.equipped
					WriteLine file, gWeaponGrenade.equipped
					WriteLine file, gWeaponParusu.equipped
					'Impulse Pulverizer
					WriteLine file, gWeaponDoubleGrenade.equipped
					WriteLine file, gWeaponSign.equipped
					WriteLine file, gWeaponAirhorn.equipped
					
					WriteLine file, gAmmoTypes[AMMO_EGG]
					WriteLine file, gAmmoTypes[AMMO_GRENADE]
					WriteLine file, gAmmoTypes[AMMO_PLASMA]
					
					WriteLine file, gKeycards[0]
					WriteLine file, gKeycards[1]
					WriteLine file, gKeycards[2]
					WriteLine file, gKeycards[3]
					
					options[optionIndex] = "SAVED!"
					optionColors[optionIndex, 0] = 1.0
					
					CloseStream(file)
			ElseIf kind = LOAD_FILES Then
				Local file:TStream = ReadFile("assets/save" + optionIndex + ".sav")
				If file <> Null Then
					gMapNumber = Int(ReadLine(file))
					gEpisodeNumber = Int(ReadLine(file))
					gGameDifficulty = Byte(ReadLine(file))
					
					gGameState = STATE_GAME
					ResetGame()
					InitGame()
					
					gWeaponChicken.equipped = Int(ReadLine(file))
					gWeaponGrenade.equipped = Int(ReadLine(file))
					gWeaponParusu.equipped = Int(ReadLine(file))
					'IP
					gWeaponDoubleGrenade.equipped = Int(ReadLine(file))
					gWeaponSign.equipped = Int(ReadLine(file))
					gWeaponAirhorn.equipped = Int(ReadLine(file))
					
					For Local i:Int = 1 To 3
						gAmmoTypes[i] = Int(ReadLine(file))
					Next
					
					For Local i:Int = 0 To 3
						gKeycards[i] = Int(ReadLine(file))
					Next
					
					CloseStream(file)
				Else
					options[optionIndex] = "NO FILE HERE!"
					optionColors[optionIndex, 0] = 1.0
					optionColors[optionIndex, 2] = 0.0
				EndIf
			EndIf
		EndIf
	End Method
	Method Enter()
		If kind = SAVE_FILES Then
			For Local i:Int = 0 To 9
				options[i] = "SAVE FILE " + String.FromInt(i)
				optionColors[i,0] = 0.0
				optionColors[i,1] = 0.0
				optionColors[i,2] = 1.0
			Next
		ElseIf kind = LOAD_FILES Then
			For Local i:Int = 0 To 9
				options[i] = "LOAD  FILE " + String.FromInt(i)
				optionColors[i,0] = 0.0
				optionColors[i,1] = 0.0
				optionColors[i,2] = 1.0
			Next
		EndIf
	End Method
	Method Leave()
		
	End Method
EndType

Global gTitleMenu:TTitleMenu = New TTitleMenu
gTitleMenu.options = ["NEW GAME", "LOAD GAME", "SAVE GAME", "OPTIONS", "CONTROLS", "EXIT"]
gTitleMenu.Init()

Global gTitleMenu2:TTitleMenu = New TTitleMenu 'For when in-game
gTitleMenu2.options = ["RESUME GAME", "NEW GAME", "LOAD GAME", "SAVE GAME", "OPTIONS", "CONTROLS", "EXIT"]
gTitleMenu2.Init()
gTitleMenu2.optionColors[0,0] = 1.0
gTitleMenu2.optionColors[0,1] = 0.0
gTitleMenu2.optionColors[0,2] = 0.0
gTitleMenu2.inGame = True

Global gOptionsMenu:TOptionsMenu = New TOptionsMenu
gOptionsMenu.options = ["SOUND VOLUME: 50%", "MUSIC VOLUME: 50%", "MOUSE SENSITIVITY: 0.3", "FOV: 90", "WINDOW:960x720", "RESOLUTION:640x480", "BACK"]
gOptionsMenu.Init()

Global gControlsMenu:TControlsMenu = New TControlsMenu
gControlsMenu.Init()

Global gEpisodeMenu:TEpisodeMenu = New TEpisodeMenu
gEpisodeMenu.options = ["1:GENOCIDE CARNIVAL","2:ENDS OF SATURN", "3:ENTER PRISMIR"]
gEpisodeMenu.Init()

Global gDifficultyMenu:TDifficultyMenu = New TDifficultyMenu
gDifficultyMenu.options = ["DON'T HOLD BACK", "I AM PREPARED", "OH NO"]
gDifficultyMenu.Init()

Global gSaveMenu:TFileMenu = New TFileMenu 
gSaveMenu.options = ["SAVE FILE 0", "SAVE FILE 1", "SAVE FILE 2", "SAVE FILE 3", "SAVE FILE 4", "SAVE FILE 5", "SAVE FILE 6", "SAVE FILE 7", "SAVE FILE 8","SAVE FILE 9", "BACK"]
gSaveMenu.Init()

Global gLoadMenu:TFileMenu = New TFileMenu 
gLoadMenu.options = ["LOAD FILE 0","LOAD FILE 1","LOAD FILE 2","LOAD FILE 3","LOAD FILE 4","LOAD FILE 5","LOAD FILE 6","LOAD FILE 7","LOAD FILE 8","LOAD FILE 9","BACK"]
gLoadMenu.Init()
gLoadMenu.kind = LOAD_FILES

Global gMenu:TMenu = gTitleMenu
Global gShowTitleGraphic:Int = True

Function SetMenu(m:TMenu)
	gMenu.Leave()
	gMenu = m
	If m = gTitleMenu And gGameInitialized = True Then
		gMenu = gTitleMenu2
	ElseIf m = gTitleMenu2 And gGameInitialized = False Then
		gMenu = gTitleMenu
	EndIf
	gMenu.Enter()
	FlushKeys()
	FlushMouse()
End Function

Function UpdateMenu()
	If KeyHit(KEY_P) Then DebugStop()
	
	SetChannelVolume gMusicChannel, sMusicVolume
	
	If gShowTitleGraphic = True Then
		If InputPressed(KEY_CONFIRM) Then 
			gShowTitleGraphic = False
			Play2DSound("menu1")
		EndIf
		FlushKeys()
		FlushMouse()
	Else
		gMenu.Update()
	EndIf
	
	If gGameState = STATE_MENU Then
		Begin2D()
		
		'Draw title graphic
		If gShowTitleGraphic = False Then 
			glColor3f 0.5, 0.5, 0.5
		Else
			glColor4f 1.0, 1.0, 1.0, 1.0
		EndIf
		glBindTexture GL_TEXTURE_2D, TTexture(MapValueForKey(gTextures, "titlegraphic")).id
		DrawQuad(0.0, 0.0, 512.0, 512.0)
			
		If gShowTitleGraphic = False Then
			gMenu.Draw()
		Else
			glColor3f 1.0, 0.5, 0.0
			DrawString("PRESS ENTER", gFont, 108.0, 176.0, 8)
		EndIf
		
		If gMenu.showTitle = True Then
			glColor4f 1.0, 1.0, 1.0, 1.0
			glBindTexture GL_TEXTURE_2D, TTexture(MapValueForKey(gTextures, "title")).id
			DrawQuad(0.0, -8.0, 512.0, 504.0)
		EndIf
		
		End2D()
		
		FitToScreen()
	EndIf
End Function

Function InputDown(index:Int)
	Return gInputs[index].IsDown()
End Function

Function InputPressed(index:Int)
	Return gInputs[index].IsPressed()
End Function

Function SaveSettings()
	Local file:TStream = WriteFile("assets/settings.txt")
	WriteLine file, sSoundVolume
	WriteLine file, sMusicVolume
	WriteLine file, sMouseSensitivity
	WriteLine file, sFov
	WriteLine file, sResNames[sWindowRes]
	WriteLine file, sResNames[sGameRes]
	For Local inp:TInput = EachIn gInputs
		If TKeyInput(inp) <> Null Then
			WriteLine file, "Key " + String(TKeyInput(inp).keyCode)
		ElseIf TMouseInput(inp) <> Null Then
			WriteLine file, "Mouse " + String(TMouseInput(inp).mouseButton)
		EndIf
	Next
	CloseStream(file)
End Function

Function LoadSettings()
	Local file:TStream = ReadFile("assets/settings.txt")
	If file <> Null Then
		sSoundVolume = Float(ReadLine(file))
		sMusicVolume = Float(ReadLine(file))
		sMouseSensitivity = Float(ReadLine(file))
		sFov = Int(ReadLine(file))
		Local wRes:String = ReadLine(file)
		For Local i:Int = 0 To Len(sResNames) - 1
			If sResNames[i] = wRes Then
				sWindowRes = i
				Exit
			EndIf
		Next
		Local gRes:String = ReadLine(file)
		For Local i = 0 To Len(sResNames) - 1
			If sResNames[i] = gRes Then
				sGameRes = i
				Exit
			EndIf
		Next
		For Local i:Int = 0 To Len(gInputs)-1
			Local n:String = ReadLine(file)
			Local parts:String[] = n.Split(" ")
			If parts[0]="Key" Then
				gInputs[i] = TKeyInput.Make(Int(parts[1]))
			ElseIf parts[0]="Mouse" Then
				gInputs[i] = TMouseInput.Make(Int(parts[1]))
			EndIf
		Next
		CloseStream(file)
	EndIf
End Function

Function NameForKeyCode:String(keyCode:Int)
	Select keyCode
		Case KEY_BACKSPACE
			Return "BACKSPACE"
		Case KEY_TAB
			Return "TAB"
		Case KEY_CLEAR
			Return "CLEAR"
		Case KEY_RETURN
			Return "RETURN"
		Case KEY_ENTER
			Return "ENTER"
		Case KEY_ESCAPE
			Return "ESCAPE"
		Case KEY_SPACE
			Return "SPACE"
		Case KEY_PAGEUP
			Return "PAGE UP"
		Case KEY_PAGEDOWN
			Return "PAGE DOWN"
		Case KEY_END
			Return "END"
		Case KEY_HOME
			Return "HOME"
		Case KEY_LEFT
			Return "LEFT"
		Case KEY_UP
			Return "UP"
		Case KEY_RIGHT
			Return "RIGHT"
		Case KEY_DOWN
			Return "DOWN"
		Case KEY_SELECT
			Return "SELECT"
		Case KEY_PRINT
			Return "PRINT"
		Case KEY_EXECUTE
			Return "EXECUTE"
		Case KEY_SCREEN
			Return "SCREEN"
		Case KEY_INSERT
			Return "INSERT"
		Case KEY_DELETE
			Return "DELETE"
		Case KEY_0
			Return "0"
		Case KEY_1
			Return "1"
		Case KEY_2
			Return "2"
		Case KEY_3
			Return "3"
		Case KEY_4
			Return "4"
		Case KEY_5
			Return "5"
		Case KEY_6
			Return "6"
		Case KEY_7
			Return "7"
		Case KEY_8
			Return "8"
		Case KEY_9
			Return "9"
		Case KEY_A
			Return "A"
		Case KEY_B
			Return "B"
		Case KEY_C
			Return "C"
		Case KEY_D
			Return "D"
		Case KEY_E
			Return "E"
		Case KEY_F
			Return "F"
		Case KEY_G
			Return "G"
		Case KEY_H
			Return "H"
		Case KEY_I
			Return "I"
		Case KEY_J
			Return "J"
		Case KEY_K
			Return "K"
		Case KEY_L
			Return "L"
		Case KEY_M
			Return "M"
		Case KEY_N
			Return "N"
		Case KEY_O
			Return "O"
		Case KEY_P
			Return "P"
		Case KEY_Q
			Return "Q"
		Case KEY_R
			Return "R"
		Case KEY_S
			Return "S"
		Case KEY_T
			Return "T"
		Case KEY_U
			Return "U"
		Case KEY_V
			Return "V"
		Case KEY_W
			Return "W"
		Case KEY_X
			Return "X"
		Case KEY_Y
			Return "Y"
		Case KEY_Z
			Return "Z"
		Case KEY_LSYS
			Return "LEFT SYSTEM"
		Case KEY_RSYS
			Return "RIGHT SYSTEM"
		Case KEY_NUM0
			Return "NUMPAD 0"
		Case KEY_NUM1
			Return "NUMPAD 1"
		Case KEY_NUM2
			Return "NUMPAD 2"
		Case KEY_NUM3
			Return "NUMPAD 3"
		Case KEY_NUM4
			Return "NUMPAD 4"
		Case KEY_NUM5
			Return "NUMPAD 5"
		Case KEY_NUM6
			Return "NUMPAD 6"
		Case KEY_NUM7
			Return "NUMPAD 7"
		Case KEY_NUM8
			Return "NUMPAD 8"
		Case KEY_NUM9
			Return "NUMPAD 9"
		Case KEY_NUMMULTIPLY
			Return "NUMPAD MULTIPLY"
		Case KEY_NUMADD
			Return "NUMPAD ADD"
		Case KEY_NUMSUBTRACT
			Return "NUMPAD SUBTRACT"
		Case KEY_NUMDECIMAL
			Return "NUMPAD DECIMAL"
		Case KEY_NUMDIVIDE
			Return "NUMPAD DIVIDE"
		Case KEY_F1
			Return "F1"
		Case KEY_F2
			Return "F2"
		Case KEY_F3
			Return "F3"
		Case KEY_F4
			Return "F4"
		Case KEY_F5
			Return "F5"
		Case KEY_F6
			Return "F6"
		Case KEY_F7
			Return "F7"
		Case KEY_F8
			Return "F8"
		Case KEY_F9
			Return "F9"
		Case KEY_F10
			Return "F10"
		Case KEY_F11
			Return "F11"
		Case KEY_F12
			Return "F12"
		Case KEY_LSHIFT
			Return "LEFT SHIFT"
		Case KEY_RSHIFT
			Return "RIGHT SHIFT"
		Case KEY_LCONTROL
			Return "LEFT CONTROL"
		Case KEY_RCONTROL
			Return "RIGHT CONTROL"
		Case KEY_LALT
			Return "LEFT ALT"
		Case KEY_RALT
			Return "RIGHT ALT"
		Case KEY_TILDE
			Return "TILDE"
		Case KEY_MINUS
			Return "MINUS"
		Case KEY_EQUALS
			Return "EQUALS"
		Case KEY_OPENBRACKET
			Return "OPEN BRACKET"
		Case KEY_CLOSEBRACKET
			Return "CLOSE BRACKET"
		Case KEY_BACKSLASH
			Return "BACKSLASH"
		Case KEY_SEMICOLON
			Return "SEMICOLON"
		Case KEY_QUOTES
			Return "QUOTE"
		Case KEY_COMMA
			Return "COMMA"
		Case KEY_PERIOD
			Return "PERIOD"
		Case KEY_SLASH
			Return "SLASH"
		Default
			Return "???"
	End Select
End Function